@@ -16,7 +16,7 @@
         MA 02110-1301, USA.
 
 
-        xfwm4    - (c) 2002-2009 Olivier Fourdan
+        xfwm4    - (c) 2002-2011 Olivier Fourdan
 
  */
 
@@ -63,7 +63,7 @@
 
 #include <gtk/gtk.h>
 #include <glib.h>
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #include "event_filter.h"
 
@@ -169,7 +169,6 @@ enum
 enum
 {
     COMPOSITING_MANAGER = 0,
-    GNOME_PANEL_DESKTOP_AREA,
     KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR,
     KWM_WIN_ICON,
     MOTIF_WM_HINTS,
@@ -250,15 +249,6 @@ enum
     PIXMAP,
     SM_CLIENT_ID,
     UTF8_STRING,
-    WIN_CLIENT_LIST,
-    WIN_DESKTOP_BUTTON_PROXY,
-    WIN_HINTS,
-    WIN_LAYER,
-    WIN_PROTOCOLS,
-    WIN_STATE,
-    WIN_SUPPORTING_WM_CHECK,
-    WIN_WORKSPACE,
-    WIN_WORKSPACE_COUNT,
     WM_CHANGE_STATE,
     WM_CLIENT_LEADER,
     WM_CLIENT_MACHINE,
@@ -290,7 +280,7 @@ struct _DisplayInfo
     GdkDisplay *gdisplay;
     Display *dpy;
 
-    SessionClient *session;
+    XfceSMClient *session;
     gboolean quit;
     gboolean reload;
 
@@ -312,7 +302,8 @@ struct _DisplayInfo
     gboolean have_xsync;
     gint shape_version;
     gint shape_event_base;
-    gint dbl_click_time;
+    gint double_click_time;
+    gint double_click_distance;
     gint xgrabcount;
     gint nb_screens;
     gchar* hostname;
@@ -360,7 +351,10 @@ struct _DisplayInfo
 DisplayInfo             *myDisplayInit                          (GdkDisplay *);
 DisplayInfo             *myDisplayClose                         (DisplayInfo *);
 gboolean                 myDisplayHaveShape                     (DisplayInfo *);
+gboolean                 myDisplayHaveShapeInput                (DisplayInfo *);
 gboolean                 myDisplayHaveRender                    (DisplayInfo *);
+void                     myDisplayCreateCursor                  (DisplayInfo *);
+void                     myDisplayFreeCursor                    (DisplayInfo *);
 Cursor                   myDisplayGetCursorBusy                 (DisplayInfo *);
 Cursor                   myDisplayGetCursorMove                 (DisplayInfo *);
 Cursor                   myDisplayGetCursorRoot                 (DisplayInfo *);
