@@ -16,7 +16,7 @@
         MA 02110-1301, USA.
 
 
-        xfwm4    - (c) 2002-2009 Olivier Fourdan
+        xfwm4    - (c) 2002-2011 Olivier Fourdan
 
  */
 
@@ -133,11 +133,6 @@ clientSetNetState (Client * c)
     XChangeProperty (display_info->dpy, c->window,
                      display_info->atoms[NET_WM_STATE], XA_ATOM, 32,
                      PropModeReplace, (unsigned char *) data, i);
-    /*
-       We also set GNOME hint here for consistency and convenience,
-       although the meaning of net_wm_state and win_state aren't the same.
-     */
-    setHint (display_info, c->window, WIN_STATE, c->win_state);
 }
 
 void
@@ -162,25 +157,21 @@ clientGetNetState (Client * c)
         if (FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
         {
             TRACE ("clientGetNetState : shaded from session management");
-            c->win_state |= WIN_STATE_SHADED;
             FLAG_SET (c->flags, CLIENT_FLAG_SHADED);
         }
         if (FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
         {
             TRACE ("clientGetNetState : sticky from session management");
-            c->win_state |= WIN_STATE_STICKY;
             FLAG_SET (c->flags, CLIENT_FLAG_STICKY);
         }
         if (FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_HORIZ))
         {
             TRACE ("clientGetNetState : maximized horiz from session management");
-            c->win_state |= WIN_STATE_MAXIMIZED_HORIZ;
             FLAG_SET (c->flags, CLIENT_FLAG_MAXIMIZED_HORIZ);
         }
         if (FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_VERT))
         {
             TRACE ("clientGetNetState : maximized vert from session management");
-            c->win_state |= WIN_STATE_MAXIMIZED_VERT;
             FLAG_SET (c->flags, CLIENT_FLAG_MAXIMIZED_VERT);
         }
     }
@@ -196,25 +187,21 @@ clientGetNetState (Client * c)
             if ((atoms[i] == display_info->atoms[NET_WM_STATE_SHADED]))
             {
                 TRACE ("clientGetNetState : shaded");
-                c->win_state |= WIN_STATE_SHADED;
                 FLAG_SET (c->flags, CLIENT_FLAG_SHADED);
             }
             else if ((atoms[i] == display_info->atoms[NET_WM_STATE_STICKY]))
             {
                 TRACE ("clientGetNetState : sticky");
-                c->win_state |= WIN_STATE_STICKY;
                 FLAG_SET (c->flags, CLIENT_FLAG_STICKY);
             }
             else if ((atoms[i] == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]))
             {
                 TRACE ("clientGetNetState : maximized horiz");
-                c->win_state |= WIN_STATE_MAXIMIZED_HORIZ;
                 FLAG_SET (c->flags, CLIENT_FLAG_MAXIMIZED_HORIZ);
             }
             else if ((atoms[i] == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]))
             {
                 TRACE ("clientGetNetState : maximized vert");
-                c->win_state |= WIN_STATE_MAXIMIZED_VERT;
                 FLAG_SET (c->flags, CLIENT_FLAG_MAXIMIZED_VERT);
             }
             else if ((atoms[i] == display_info->atoms[NET_WM_STATE_FULLSCREEN]))
@@ -283,6 +270,43 @@ clientGetNetState (Client * c)
     }
 }
 
+void
+clientUpdateNetWmDesktop (Client * c, XClientMessageEvent * ev)
+{
+    ScreenInfo *screen_info;
+
+    g_return_if_fail (c != NULL);
+    TRACE ("entering clientUpdateNetWmDesktop");
+    TRACE ("client \"%s\" (0x%lx), value 0x%lx", c->name, c->window, ev->data.l[0]);
+
+    screen_info = c->screen_info;
+
+    if ((guint) ev->data.l[0] == ALL_WORKSPACES)
+    {
+        if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_STICK) && !FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
+        {
+            clientStick (c, TRUE);
+            frameQueueDraw (c, FALSE);
+        }
+    }
+    else if ((guint) ev->data.l[0] < (guint) screen_info->workspace_count)
+    {
+        if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_STICK) && FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
+        {
+            clientUnstick (c, TRUE);
+            frameQueueDraw (c, FALSE);
+        }
+        if ((guint) ev->data.l[0] != (guint) c->win_workspace)
+        {
+            clientSetWorkspace (c, (guint) ev->data.l[0], TRUE);
+        }
+    }
+    else
+    {
+        TRACE ("Ignoring invalid NET_WM_DESKTOP value 0x%lx specified for client \"%s\" (0x%lx)", ev->data.l[0], c->name, c->window);
+    }
+}
+
 void
 clientUpdateNetState (Client * c, XClientMessageEvent * ev)
 {
@@ -351,16 +375,16 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
         {
             if ((action == NET_WM_STATE_ADD) && !FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED))
             {
-                mode = 0;
+                mode = 0L;
                 if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]))
                 {
-                    mode |= !FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_HORIZ) ? WIN_STATE_MAXIMIZED_HORIZ : 0;
+                    mode |= !FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_HORIZ) ? CLIENT_FLAG_MAXIMIZED_HORIZ : 0;
                 }
                 if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]))
                 {
-                    mode |= !FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_VERT) ? WIN_STATE_MAXIMIZED_VERT : 0;
+                    mode |= !FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_VERT) ? CLIENT_FLAG_MAXIMIZED_VERT : 0;
                 }
                 clientToggleMaximized (c, mode, TRUE);
             }
@@ -370,12 +394,12 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
                 if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]))
                 {
-                    mode |= FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_HORIZ) ? WIN_STATE_MAXIMIZED_HORIZ : 0;
+                    mode |= FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_HORIZ);
                 }
                 if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]))
                 {
-                    mode |= FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_VERT) ? WIN_STATE_MAXIMIZED_VERT : 0;
+                    mode |= FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_VERT);
                 }
                 clientToggleMaximized (c, mode, TRUE);
             }
@@ -385,12 +409,12 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
                 if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]))
                 {
-                    mode |= WIN_STATE_MAXIMIZED_HORIZ;
+                    mode |= CLIENT_FLAG_MAXIMIZED_HORIZ;
                 }
                 if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]))
                 {
-                    mode |= WIN_STATE_MAXIMIZED_VERT;
+                    mode |= CLIENT_FLAG_MAXIMIZED_VERT;
                 }
                 clientToggleMaximized (c, mode, TRUE);
             }
@@ -792,9 +816,6 @@ clientGetNetWmType (Client * c)
             case WIN_LAYER_DOCK:
                 c->type_atom = display_info->atoms[NET_WM_WINDOW_TYPE_DOCK];
                 break;
-            case WIN_LAYER_NORMAL:
-                c->type_atom = display_info->atoms[NET_WM_WINDOW_TYPE_NORMAL];
-                break;
             default:
                 if (c->transient_for != None)
                 {
@@ -865,7 +886,6 @@ clientGetInitialNetWmDesktop (Client * c)
             {
                 TRACE ("atom net_wm_desktop specifies window \"%s\" is sticky", c->name);
                 FLAG_SET (c->flags, CLIENT_FLAG_STICKY);
-                c->win_state |= WIN_STATE_STICKY;
             }
             c->win_workspace = c->screen_info->current_ws;
         }
@@ -876,12 +896,6 @@ clientGetInitialNetWmDesktop (Client * c)
         }
         FLAG_SET (c->xfwm_flags, XFWM_FLAG_WORKSPACE_SET);
     }
-    else if (getHint (display_info, c->window, WIN_WORKSPACE, &val))
-    {
-        TRACE ("atom win_workspace specifies window \"%s\" is on desk %i", c->name, (int) val);
-        c->win_workspace = (int) val;
-        FLAG_SET (c->xfwm_flags, XFWM_FLAG_WORKSPACE_SET);
-    }
 
     /* This is to make sure that transient are shown with their "ancestor" window */
     if (!FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
@@ -894,7 +908,6 @@ clientGetInitialNetWmDesktop (Client * c)
             if (FLAG_TEST (c2->flags, CLIENT_FLAG_STICKY))
             {
                 FLAG_SET (c->flags, CLIENT_FLAG_STICKY);
-                c->win_state |= WIN_STATE_STICKY;
                 c->win_workspace = c->screen_info->current_ws;
             }
         }
@@ -908,7 +921,6 @@ clientGetInitialNetWmDesktop (Client * c)
         FLAG_SET (c->xfwm_flags, XFWM_FLAG_WORKSPACE_SET);
     }
     TRACE ("initial desktop for window \"%s\" is %i", c->name, c->win_workspace);
-    setHint (display_info, c->window, WIN_WORKSPACE, c->win_workspace);
     if (FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
     {
         setHint (display_info, c->window, NET_WM_DESKTOP, (unsigned long) ALL_WORKSPACES);
@@ -957,12 +969,10 @@ clientValidateNetStrut (Client * c)
 {
     ScreenInfo *screen_info;
     gboolean valid;
-    int max_value;
 
     g_return_val_if_fail (c != NULL, TRUE);
     TRACE ("entering clientValidateNetStrut for \"%s\" (0x%lx)", c->name, c->window);
     screen_info = c->screen_info;
-    max_value = MIN (screen_info->width, screen_info->height) / 4;
     valid = TRUE;
 
     if (c->struts[STRUTS_TOP] > screen_info->height - screen_info->margins[STRUTS_BOTTOM])
@@ -1198,7 +1208,6 @@ clientWindowType (Client * c)
             TRACE ("atom net_wm_window_type_desktop detected");
             c->type = WINDOW_DESKTOP;
             c->initial_layer = WIN_LAYER_DESKTOP;
-            c->win_state |= WIN_STATE_STICKY;
             FLAG_SET (c->flags,
                 CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_STICKY |
                 CLIENT_FLAG_SKIP_TASKBAR);
@@ -1251,8 +1260,10 @@ clientWindowType (Client * c)
             c->type = WINDOW_DIALOG;
             c->initial_layer = WIN_LAYER_NORMAL;
             /* Treat DIALOG without transient_for set as transient for group */
-            if (c->transient_for == None)
+            if ((c->transient_for == None) || (!clientGetTransient (c)))
             {
+                TRACE ("Invalid transient 0x%lx specified for dialog window 0x%lx (%s)",
+                       c->transient_for, c->window, c->name);
                 c->transient_for = c->screen_info->xroot;
             }
         }
@@ -1270,8 +1281,10 @@ clientWindowType (Client * c)
             c->type = WINDOW_UTILITY;
             c->initial_layer = WIN_LAYER_NORMAL;
             /* Treat UTILITY without transient_for set as transient for group */
-            if (c->transient_for == None)
+            if ((c->transient_for == None) || (!clientGetTransient (c)))
             {
+                TRACE ("Invalid transient 0x%lx specified for utility window 0x%lx (%s)",
+                       c->transient_for, c->window, c->name);
                 c->transient_for = c->screen_info->xroot;
             }
         }
@@ -1305,7 +1318,7 @@ clientWindowType (Client * c)
         if ((c2) && (c->initial_layer < c2->win_layer))
         {
             c->initial_layer = c2->win_layer;
-            TRACE ("Applied layer is %i", c->initial_layer);
+            TRACE ("Applied layer is %lu", c->initial_layer);
         }
         FLAG_UNSET (c->xfwm_flags, XFWM_FLAG_HAS_HIDE);
     }
@@ -1313,7 +1326,7 @@ clientWindowType (Client * c)
     {
         if ((old_type != c->type) || (c->initial_layer != c->win_layer))
         {
-            TRACE ("setting layer %i", c->initial_layer);
+            TRACE ("setting layer %lu", c->initial_layer);
             clientSetLayer (c, c->initial_layer);
             clientSetNetState (c);
         }
@@ -1379,7 +1392,11 @@ clientHandleNetActiveWindow (Client *c, guint32 timestamp, gboolean source_is_ap
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
     ev_time = myDisplayGetTime (display_info, timestamp);
-
+    if (!source_is_application && (timestamp == 0))
+    {
+         TRACE ("Client \"%s\" (0x%lx) sent a NET_ACTIVE_WINDOW message with a timestamp of 0", c->name, c->window);
+         source_is_application = TRUE;
+    }
     if (source_is_application)
     {
         current_time = myDisplayGetLastUserTime (display_info);
@@ -1444,7 +1461,7 @@ void
 clientReceiveNetWMPong (ScreenInfo *screen_info, guint32 timestamp)
 {
     Client *c;
-    int i;
+    guint i;
 
     g_return_if_fail (screen_info != NULL);
     g_return_if_fail (timestamp != CurrentTime);
