@@ -17,7 +17,7 @@
 
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2009 Olivier Fourdan
+        xfwm4    - (c) 2002-2011 Olivier Fourdan
 
  */
 
@@ -92,11 +92,18 @@ enum
     KEY_POPUP_MENU,
     KEY_PREV_WORKSPACE,
     KEY_RAISE_WINDOW,
+    KEY_RAISELOWER_WINDOW,
     KEY_RESIZE,
     KEY_RIGHT_WORKSPACE,
     KEY_SHADE_WINDOW,
     KEY_SHOW_DESKTOP,
     KEY_STICK_WINDOW,
+    KEY_SWITCH_APPLICATION,
+    KEY_SWITCH_WINDOW,
+    KEY_TILE_DOWN,
+    KEY_TILE_LEFT,
+    KEY_TILE_RIGHT,
+    KEY_TILE_UP,
     KEY_TOGGLE_ABOVE,
     KEY_TOGGLE_FULLSCREEN,
     KEY_UP_WORKSPACE,
@@ -131,11 +138,11 @@ enum
 
 enum
 {
-    DBL_CLICK_ACTION_NONE,
-    DBL_CLICK_ACTION_MAXIMIZE,
-    DBL_CLICK_ACTION_SHADE,
-    DBL_CLICK_ACTION_FILL,
-    DBL_CLICK_ACTION_HIDE
+    DOUBLE_CLICK_ACTION_NONE,
+    DOUBLE_CLICK_ACTION_MAXIMIZE,
+    DOUBLE_CLICK_ACTION_SHADE,
+    DOUBLE_CLICK_ACTION_FILL,
+    DOUBLE_CLICK_ACTION_HIDE
 };
 
 enum
@@ -163,12 +170,12 @@ struct _XfwmParams
 {
     MyKey keys[KEY_COUNT];
     gchar button_layout[BUTTON_STRING_COUNT + 1];
-    unsigned int xfwm_margins[4];
+    int xfwm_margins[4];
     int activate_action;
     int button_offset;
     int button_spacing;
     int double_click_action;
-    int easy_click;
+    guint easy_click;
     int focus_delay;
     int frame_opacity;
     int inactive_opacity;
@@ -179,7 +186,6 @@ struct _XfwmParams
     int popup_opacity;
     int raise_delay;
     int resize_opacity;
-    int restore_on_move;
     int shadow_delta_height;
     int shadow_delta_width;
     int shadow_delta_x;
@@ -194,6 +200,7 @@ struct _XfwmParams
     gboolean box_move;
     gboolean box_resize;
     gboolean click_to_focus;
+    gboolean cycle_apps_only;
     gboolean cycle_draw_frame;
     gboolean cycle_hidden;
     gboolean cycle_minimum;
@@ -201,11 +208,13 @@ struct _XfwmParams
     gboolean focus_hint;
     gboolean focus_new;
     gboolean full_width_title;
+    gboolean mousewheel_rollup;
     gboolean prevent_focus_stealing;
     gboolean raise_on_click;
     gboolean raise_on_focus;
     gboolean raise_with_any_button;
     gboolean repeat_urgent_blink;
+    gboolean restore_on_move;
     gboolean scroll_workspaces;
     gboolean show_app_icon;
     gboolean show_dock_shadow;
@@ -214,6 +223,7 @@ struct _XfwmParams
     gboolean snap_resist;
     gboolean snap_to_border;
     gboolean snap_to_windows;
+    gboolean tile_on_move;
     gboolean title_vertical_offset_active;
     gboolean title_vertical_offset_inactive;
     gboolean toggle_workspaces;
